"""gdc

Tool-specific initialization for the GNU D compiler.

Created by Jerome M. Berger <jeberger@free.fr>
28 May 2007

This script was largely inspired from the dmd.py script that comes
with SCons. It requires an environment that has already been set up
for the GNU tools (especially for the linker).

Compiler variables:
    DC - The name of the D compiler to use. Defaults to gdc;
    DPATH - List of paths to search for import modules;
    DFLAGS - Extra compiler flags;
    DVERSIONS - List of version tags to enable when compiling;
    DDEBUG - List of debug tags to enable when compiling.
"""

import os
import string

import SCons.Action
import SCons.Builder
import SCons.Defaults
import SCons.Tool
import SCons.Util

############################################################
# Taken from dmd.py
def isD (source):
    if not source:
        return 0

    for s in source:
        if s.sources:
            ext = os.path.splitext(str(s.sources[0]))[1]
            if ext == '.d':
                return 1
    return 0
############################################################

############################################################
# Taken from D.py
#
# Note that the associated patches have been sent upstream and
# integrated in the development versions of SCons.
def DScanner():
    """Return a prototype Scanner instance for scanning D source files"""
    ds = D(name = "DScanner",
           suffixes = '$DSUFFIXES',
           path_variable = 'DPATH',
           regex = 'import\s+(?:[a-zA-Z0-9_.]+)\s*(?:,\s*(?:[a-zA-Z0-9_.]+)\s*)*;')
    return ds

class D(SCons.Scanner.Classic):
    import re
    cre2 = re.compile ('(?:import\s)?\s*([a-zA-Z0-9_.]+)\s*(?:,|;)', re.M)

    def find_include(self, include, source_dir, path):
        # translate dots (package separators) to slashes
        inc = string.replace(include, '.', '/')

        i = SCons.Node.FS.find_file (inc + '.d', (source_dir,) + path)
        # +++ JB +++
        if (i is None):
            i = SCons.Node.FS.find_file (inc + '.di', (source_dir,) + path)
        # --- JB ---
        return i, include

    def find_include_names (self, node):
        includes = []
        for i in self.cre.findall (node.get_contents()):
            includes += self.cre2.findall (i)
        return includes

    #-------------------------------------------------------
    # Taken from SCons.Scanner.Classic
    def scan(self, node, path=()):

        # cache the includes list in node so we only scan it once:
        if node.includes != None:
            includes = node.includes
        else:
            # +++ JB +++
            includes = self.find_include_names (node)
            # --- JB ---
            node.includes = includes

        # This is a hand-coded DSU (decorate-sort-undecorate, or
        # Schwartzian transform) pattern.  The sort key is the raw name
        # of the file as specifed on the #include line (including the
        # " or <, since that may affect what file is found), which lets
        # us keep the sort order constant regardless of whether the file
        # is actually found in a Repository or locally.
        nodes = []
        source_dir = node.get_dir()
        if callable(path):
            path = path()
        for include in includes:
            n, i = self.find_include(include, source_dir, path)

            if n is None:
                SCons.Warnings.warn(SCons.Warnings.DependencyWarning,
                                    "No dependency generated for file: %s (included from: %s) -- file not found" % (i, node))
            else:
                sortkey = self.sort_key(include)
                nodes.append((sortkey, n))

        nodes.sort()
        nodes = map(lambda pair: pair[1], nodes)
        return nodes
    #-------------------------------------------------------

############################################################

def generate(env):
    static_obj, shared_obj = SCons.Tool.createObjBuilders(env)

    DAction = SCons.Action.Action('$DCOM', '$DCOMSTR')

    static_obj.add_action('.d', DAction)
    shared_obj.add_action('.d', DAction)
    static_obj.add_emitter('.d', SCons.Defaults.StaticObjectEmitter)
    shared_obj.add_emitter('.d', SCons.Defaults.SharedObjectEmitter)

    scanner = DScanner()
    static_obj.source_scanner.add_scanner (".d", scanner)
    static_obj.source_scanner.add_scanner (".di", scanner)
    shared_obj.source_scanner.add_scanner (".d", scanner)
    shared_obj.source_scanner.add_scanner (".di", scanner)

    env['DC'] = 'gdc'
    env['DCOM'] = '$DC $_DINCFLAGS $_DVERFLAGS $_DDEBUGFLAGS $DFLAGS -c -o $TARGET $SOURCES'
    env['_DINCFLAGS'] = '${_concat(DINCPREFIX, DPATH, DINCSUFFIX, __env__, RDirs, TARGET, SOURCE)}'
    env['_DVERFLAGS'] = '${_concat(DVERPREFIX, DVERSIONS, DVERSUFFIX, __env__)}'
    env['_DDEBUGFLAGS'] = '${_concat(DDEBUGPREFIX, DDEBUG, DDEBUGSUFFIX, __env__)}'

    env['DPATH'] = ['#/']
    env['DFLAGS'] = []
    env['DVERSIONS'] = []
    env['DDEBUG'] = []

    # Add the path to the standard library.
    # This is merely for the convenience of the dependency scanner.
    gdcPath = env.WhereIs ('gdc')
    if gdcPath:
        phobosDir = os.path.join (os.path.dirname (gdcPath),
                                  '/../include/d/%s' % os.popen ("gdc -dumpversion").read().strip())
        if os.path.isdir (phobosDir):
            env.Append (DPATH = [ phobosDir ])

    env['DINCPREFIX'] = '-I'
    env['DINCSUFFIX'] = ''
    env['DVERPREFIX'] = '-fversion='
    env['DVERSUFFIX'] = ''
    env['DDEBUGPREFIX'] = '-fdebug='
    env['DDEBUGSUFFIX'] = ''
    env['DFLAGPREFIX'] = ''
    env['DFLAGSUFFIX'] = ''
    env['DFILESUFFIX'] = '.d'

    def smartLink (source, target, env, for_signature,
                   default = env.get ("SMARTLINK", env.get ("LINK"))):
        if isD (source):
            return '$DC'
        return default

    env["SMARTLINK"] = smartLink
