# Each target should define the following environment variables:
#
# ``DATA_PATH``
#       Default path in which program data should be installed;
#
# ``HEADER_PATH``
#       Default path in which header files should be installed;
#
# ``LIBRARY_PATH``
#       Default path in which libraries should be installed;
#
# ``PROGRAM_PATH``
#       Default path in which executables should be installed.
#
# In addition, a target may  define the variables ``DATA_POSTACTION``,
# ``HEADER_POSTACTION``,           ``LIBRARY_POSTACTION``          and
# ``PROGRAM_POSTACTION`` to  tell SCons to   execute a specific action
# after each install command has been executed.

import SCons.Environment
import sconstools as tools

# Wrap the Install and InstallAs builder methods so that they will
# update the "install" target
def makeInstallWrapper (method):
    def install (env, *args):
        result = method (env, *args)
        env.Alias ("install", result)
        return result
    return install

# Install the sources to the corresponding targets as defined by the
# environment variables.
def makePathInstallWrapper (name):
    def install (env, path, nodes = None):
        if nodes is None:
            nodes = path
            path  = None
        filt = env.get (install.filter, None)
        if filt:
            toInstall = filt (env, nodes)
        else:
            toInstall = { install.basePath: nodes }
        result = []
        if path:
            import os
            for base, nodes in toInstall.items():
                result.append (env.Install (os.path.normpath ("%s/%s" % (base, path)),
                                            nodes))
        else:
            for base, nodes in toInstall.items():
                result.append (env.Install (base, nodes))
        action = env.get (install.postaction, None)
        if action:
            import SCons.Action
            if isinstance (action, SCons.Action.ActionBase):
                env.AddPostAction (result, action)
            elif action.needed (env, result):
                env.AddPostAction (result, action.action)
        return env.Flatten (result)
    install.basePath   = "${DEST_DIR}${%s_PATH}" % name
    install.filter     = "%s_FILTER" % name
    install.postaction = "%s_POSTACTION" % name
    return install

def register (env, vars):
    env.AddMethod (makeInstallWrapper (env.Install.method), "Install")
    env.AddMethod (makeInstallWrapper (env.InstallAs.method), "InstallAs")

    env.AddMethod (makePathInstallWrapper ("DATA"), "InstallData")
    env.AddMethod (makePathInstallWrapper ("DOC"), "InstallDoc")
    env.AddMethod (makePathInstallWrapper ("HEADER"), "InstallHeader")
    env.AddMethod (makePathInstallWrapper ("LIBRARY"), "InstallLibrary")
    env.AddMethod (makePathInstallWrapper ("PROGRAM"), "InstallProgram")

    vars.AddVariables (("data_path",
                        "Path in which program data should be installed",
                        env["DATA_PATH"]),
                       ("doc_path",
                        "Path in which documentation should be installed",
                        env["DOC_PATH"]),
                       ("header_path",
                        "Path in which header files should be installed",
                        env["HEADER_PATH"]),
                       ("library_path",
                        "Path in which libraries should be installed",
                        env["LIBRARY_PATH"]),
                       ("program_path",
                        "Path in which executable programs should be installed",
                        env["PROGRAM_PATH"]))
    vars.Update (env)

    for path in ("data_path", "header_path", "library_path", "program_path"):
        val = env[path]
        if val != '':
            env[path.upper()] = val
