import os
import re
import sconstools as tools

def setCommonVariables (env):
    consVars = [ "CCFLAGS", "CPPPATH", "CPPDEFINES",
                 "LIBS", "LIBPATH", "LINKFLAGS" ]
    if env["optim"] == "debug":
        for var in consVars:
            env[var] = [ "$%s_CMN" % var, "$%s_DBG" % var ]
        env.Append (CPPDEFINES = [ "DEBUG" ])
    elif env["optim"] == "speed":
        for var in consVars:
            env[var] = [ "$%s_CMN" % var, "$%s_SPD" % var ]
        env.Append (CPPDEFINES = [ "NDEBUG" ])
    elif env["optim"] == "size":
        for var in consVars:
            env[var] = [ "$%s_CMN" % var, "$%s_SIZ" % var ]
        env.Append (CPPDEFINES = [ "NDEBUG" ])
    elif env["optim"] != "custom":
        raise Exception, "Unknown optimization variable!"

    env.Append (CPPDEFINES = [ "WIN32" ])

    # Change the shared object suffix to avoid name collisions.
    env["SHOBJSUFFIX"] = ".os"

def setMinGWVariables (env, vars):
    print "Setting variables for MinGW"
    env["CCFLAGS_CMN"] = [ "-Wall", "--pedantic", "-Wextra", "-Wno-long-long", "-pipe" ]
    env["CCFLAGS_ANS"] = [ "-ansi" ]
    env["CCFLAGS_DBG"] = [ "-O0", "-g" ]
    env["CCFLAGS_SIZ"] = [ "-Os", "-fomit-frame-pointer", "$CCFLAGS_TUN", "$CCFLAGS_ARCH" ]
    env["CCFLAGS_SPD"] = [ "-O2", "-fomit-frame-pointer", "$CCFLAGS_TUN", "$CCFLAGS_ARCH" ]

    env["LINKFLAGS_CMN"] = [ "$_RPATH" ]
    env["LINKFLAGS_DBG"] = []
    env["LINKFLAGS_SIZ"] = [ "-s" ]
    env["LINKFLAGS_SPD"] = [ "-s" ]
    env["LINKFLAGS_WIN"] = [ "-mwindows" ]

    setCommonVariables (env)

    vars.AddVariables (("tune", "CPU model to tune for (will still work on older CPUs)"),
                       ("arch", "CPU architecture to compile for (won't work on older CPUs)"))
    vars.Update (env)
    tune = env.get ("tune", "")
    if tune != "":
        env["CCFLAGS_TUN"] = "-mtune=%s" % tune
    arch = env.get ("arch", "")
    if arch != "":
        env["CCFLAGS_ARCH"] = "-march=%s" % arch

    # Remove the .exp file from the list of targets
    def shlib_emitter (target, source, env):
        exp = env.FindIxes (target, 'WINDOWSEXPPREFIX', 'WINDOWSEXPSUFFIX')
        if exp:
            target.remove (exp)
        return target, source
    env.Append (SHLIBEMITTER = [ shlib_emitter ])

    # Setup the mpatrol memory debugger
    import sconstools
    sconstools.setupMPatrol (vars, env, [ "bfd", "iberty", "imagehlp", "intl" ])

def setMSVCVariables (env):
    print "Setting variables for Visual Studio"
    env["CCFLAGS_CMN"] = [ "/nologo",   # Suppress banner
                           "/W3"        # Warning level 3
                           ]
    env["CCFLAGS_ANS"] = [ "/Za" ]
    env["CCFLAGS_DBG"] = [ "/MTd",      # Use debug multi threaded libc
                           "/Od",       # Optimization level: debug
                           "/Z7"        # Include debug information in the object files
                           ]
    env["CCFLAGS_SIZ"] = [ "/MT",       # Use release multi threaded libc
                           "/O1"        # Optimization level: 1 (size)
                           ]
    env["CCFLAGS_SPD"] = [ "/MT",       # Use release multi threaded libc
                           "/O2"        # Optimization level: 2 (speed)
                           ]

    # Handle deprecated variables
    matcher = re.compile (r"Version ([0-9]+)\.[0-9]+\.[0-9]+")
    pipe = os.popen4 (env["CC"])[1]
    version = 0
    for line in pipe.readlines():
        match = re.search (matcher, line)
        if match:
            version = int (match.group (1))

    if version < 13:
        if version == 0:
            tools.printMessage ("WARNING",
                                "Unable to determine the VisualC version, assuming version 6.")
        # Assume VC6
        env["CCFLAGS_CMN"].append ("/GX") # Enable exception handling
    else:
        env["CCFLAGS_CMN"].append ("/EHsc") # Enable exception handling

    setCommonVariables (env)

def setVariables (env, vars):
    # Check if mingw is available
    useMinGW = False
    if env.WhereIs ("gcc"):
        matcher = re.compile ("mingw", re.I)
        pipe = os.popen3 ("gcc -v")[2]
        for line in pipe.readlines():
            if re.search (matcher, line):
                useMinGW = True
                break
        pipe.close()

    # Add a command line variable to override the defaults
    from SCons.Variables import BoolVariable, EnumVariable
    vars.AddVariables (EnumVariable ("toolchain",
                                     "Select toolchain to use",
                                     "default",
                                     [ "default", "mingw", "msvc" ]),
                       BoolVariable ("posix_paths",
                                     "Should we use a posix-like directory structure?",
                                     False))
    vars.Update (env)

    if env["toolchain"] == "msvc":
        useMinGW = False

    if useMinGW:
        env.Tool ("mingw")

    if re.search ("gcc(\\.exe)?$", env["CC"], re.I):
        setMinGWVariables (env, vars)
    elif re.search ("cl(\\.exe)?$", env["CC"], re.I):
        setMSVCVariables (env)
    else:
        print "WARNING: Unknown native win32 compiler"

    # Install DLLs alongside executables
    def filterInstalledLibs (env, nodes):
        result = {}
        for n in nodes:
            dest = "${PROGRAM_PATH}" if str (n).endswith (env["SHLIBSUFFIX"]) else "${LIBRARY_PATH}"
            sources = result.get (dest, None)
            if sources:
                sources.append (n)
            else:
                result[dest] = [ n ]
        return result
    env["LIBRARY_FILTER"] = filterInstalledLibs

    if env["posix_paths"]:
        env["DATA_PATH"]      = "${PREFIX}/share/${PACKAGE}"
        env["DOC_PATH"]       = "${PREFIX}/share/doc/${PACKAGE}"
        env["HEADER_PATH"]    = "${PREFIX}/include/${PACKAGE}"
        env["LIBRARY_PATH"]   = "${PREFIX}/lib"
        env["PROGRAM_PATH"]   = "${PREFIX}/bin"
    else:
        env["DATA_PATH"]    = "${PREFIX}/data"
        env["DOC_PATH"]     = "${PREFIX}/doc"
        env["HEADER_PATH"]  = "${PREFIX}/include/${PACKAGE}"
        env["LIBRARY_PATH"] = "${PREFIX}"
        env["PROGRAM_PATH"] = "${PREFIX}"
