##
# Add a target to generate html files from a ReStructured Text file.
#
# Example:
#
# ================ BEGIN SConscript snippet ===============
# import sconstools.rst2html
# sconstools.rst2html.register (env)
#
# env.Rst2Html ("readme.html", "readme.txt")
# ================= END SConscript snippet ================
#
# If you have  installed   the Pygments syntax highlighter,  you   can
# create  highlighted source code snippets in  your RST files with the
# code-block directive. For example:
#
# ================ BEGIN readme.txt snippet ===============
# Here is some Python code:
#
# code-block:: python
#
#     print "Hello World"
#
# And here is the same in C:
#
# code-block:: c
#
#     printf ("Hello World\n");
#
# ================= END readme.txt snippet ================
#
# If the Pygments syntax highlighter is not available, this will still
# work  and the  code-block directive  will be   equivalent to  an RST
# literal block.
#
# Note that this  tool requires a  CSS stylesheet named "docutils.css"
# to be  put in the  same path as the HTML  file.  There are currently
# three predefined  stylesheets with the   sconstools and docutils and
# you can get the path  to the current  version  of these through  the
# following variables:
#
# - docutilsCss:    The default Docutils stylesheet;
# - pygmentsCss:    A stylesheet    for    code  highlighting   (named
#                   "pygments.css");
# - sconstoolsCss:  The default Sconstools stylesheet. It requires the
#                   other two.
import os
import sconstools as tools
import sys

######################################################################
# Setup the code-block directive
######################################################################
try:
    import locale
    locale.setlocale(locale.LC_ALL, '')
except:
    pass

import docutils.parsers.rst

from docutils import nodes
from docutils.parsers.rst import directives

try:
    from pygments import highlight
    from pygments.lexers import get_lexer_by_name
    from pygments.formatters import HtmlFormatter

    __pygments_formatter = HtmlFormatter()

    def pygments_directive (name, arguments, options, content, lineno,
                            content_offset, block_text, state, state_machine):
        try:
            lexer = get_lexer_by_name (arguments[0])
        except ValueError:
            # no lexer found - use the text one instead of raising an
            # exception
            lexer = get_lexer_by_name ('text')
        parsed = highlight (u'\n'.join (content), lexer, __pygments_formatter)
        return [ nodes.raw ('', parsed, format='html') ]

except:
    def pygments_directive (name, arguments, options, content, lineno,
                            content_offset, block_text, state, state_machine):
        parsed = u'\n'.join (content)
        return [ nodes.literal_block (parsed, parsed) ]

pygments_directive.arguments = (1, 0, 1)
pygments_directive.content = 1
directives.register_directive('code-block', pygments_directive)

######################################################################
# Configure the SCons environment
######################################################################

## Builder function
def __build (env, target, source):
    import docutils.core
    docutils.core.publish_file (source_path = str (source[0]),
                                destination_path = str (target[0]),
                                writer_name = 'html',
                                settings_overrides = { "embed_stylesheet": False,
                                                       "stylesheet": "docutils.css",
                                                       "stylesheet_path": None,
                                                       "_disable_config": True })

##
# Adds the following items to a SCons environment:
# - An "Rst2Html" builder which will  take a "foo.txt" or a "foot.rst"
#   file and converts it into a "foo.html".
def register (env):
    # Add the Subst builder
    a = env.Action (__build, "$$RST2HTMLSTR")
    b = env.Builder (action = a,
                     src_suffix = [ ".rst", ".txt" ],
                     suffix = ".html",
                     single_source = True)
    env.Append (BUILDERS = { "Rst2Html": b })
    if env["verbose"]:
        env["RST2HTMLSTR"] = tools.makeVerboseCommandString ("Compiling ReStructured Text to ${TARGET.file} with:",
                                                             "Subst (${TARGET}, ${SOURCE})")
    else:
        env["RST2HTMLSTR"] = tools.makeStandardCommandString ("Compiling ReStructured Text to ${TARGET.file}...")

import docutils.writers.html4css1
docutilsCss = os.path.join (docutils.writers.html4css1.__path__[0], "html4css1.css")
if not os.path.exists (docutilsCss):
    tools.printMessage ("WARNING",
                        "The docutils style sheet is not in the expected location!\n"
                        r"Will assume that we are on Windows and must remove the '\lib\site-package' path component")
    docutilsCss = docutilsCss.replace (r'\lib\site-packages', '', 1)
sconstoolsCss = os.path.join (sys.prefix, "share", "sconstools", "docutils.css")
pygmentsCss = os.path.join (sys.prefix, "share", "sconstools", "pygments.css")
