##
# Add a   target to generate  text  files by  substituting environment
# variables into its input.
#
# Example:
#
# =================== BEGIN config.h.in ===================
# /* Package name */
# #define PACKAGE "%(PACKAGE)s"
#
# /* Define if stdint.h is present. */
# %(HAVE_STDINT_H)s
# ==================== END config.h.in ====================
#
# ================ BEGIN SConscript snippet ===============
# import sconstools.subst
# sconstools.subst.register (env)
#
# env["PACKAGE"] = "foo"
# if haveStdintH:
#     env.Defined ("HAVE_STDINT_H")
# else:
#     env.Undefined ("HAVE_STDINT_H")
#
# env.Subst ("config.h", "config.h.in")
# ================= END SConscript snippet ================
import re
import sconstools as tools

## Builder function
def __build (env, target, source):
    input = file (str (source[0]), "r")
    output = file (str (target[0]), "w")
    regexp = env.get ("SUBST_REGEXP", None)
    if regexp is None:
        class Wrapper:
            def __init__ (self, env):
                self.env = env
            def __getitem__ (self, key):
                return self.env.subst ("${%s}" % key)
        dict = Wrapper (env)
        for l in input.readlines():
            output.write (l % dict)
    else:
        if not hasattr (regexp, "sub"):
            regexp = re.compile (regexp)
        dict = env.Dictionary()
        def sub (match):
            return dict[match.group (1)]
        for l in input.readlines():
            output.write (regexp.sub (sub, l))
            
    input.close()
    output.close()

## Emitter function (will register the dependencies)
def __emit (target, source, env):
    regexp = env.get ("SUBST_REGEXP", None)
    if regexp is None:
        start = re.compile (r"%\(")
        paren = re.compile ("[()]")
        word  = re.compile (r"\w+")
    for src in source:
        values = set()
        if regexp is None:
            src      = src.get_contents()
            l        = len (src)
            i        = 0
            nbParens = 0
            sMatch   = start.search (src)
            while sMatch:
                i        = sMatch.end()
                pMatch   = paren.search (src, i)
                nbParens = 0
                while pMatch:
                    if pMatch.group() == ')':
                        if nbParens == 0:
                            # This ')' matches the opening '%(' -> the
                            # targets depend on any variable between
                            # the two.
                            for wMatch in word.findall (src, i, pMatch.start()):
                                for tgt in target:
                                    env.Depends (tgt, env.Value (env.get (wMatch, "")))
                            break
                        else:
                            # This ')' matches a '(' inside the
                            # substitution pattern -> simply keep
                            # track.
                            nbParens -= 1
                    else:
                        nbParens += 1
                    pMatch = paren.search (src, pMatch.end())
                sMatch = start.search (src, pMatch.end()) if pMatch else None
        else:
            if not hasattr (regexp, "findall"):
                regexp = re.compile (regexp)
            for match in regexp.findall (src.get_contents()):
                for tgt in target:
                    env.Depends (tgt, env.Value (env[match]))
    return (target, source)

## Wrapper class for "Defined" values
def __Defined (env, name, value = None):
    class Defined:
        def __init__ (self, name, value):
            self.name  = name
            self.value = value

        def __str__ (self):
            if self.value is None:
                return "#define %s" % self.name
            else:
                return "#define %s %s" % (self.name, self.value)
    env[name] = Defined (name, value)

## Wrapper class for "Undefined" values
def __Undefined (env, name):
    class Undefined:
        def __init__ (self, name):
            self.name = name

        def __str__ (self):
            return "/* #undef %s */" % self.name
    env[name] = Undefined (name)

## Automatically "Define" or "Undefine" a value depending on a test
def CheckDefined (env, test, name, value = None):
    if test:
        return env.Defined (name, value)
    else:
        return env.Undefined (name)

##
# Adds the following items to a SCons environment:
# - A "Subst" builder  which will take  a "foo.in" file and convert it
#   into a  "foo" file while  performing  variable substitution on  it
#   (using the environment values for substitution variables);
# - A "Defined" function that will create an environment variable with
#   the proper substitution for a C/C++ #define statement;
# - An "Undefined" function that  will create an environment  variable
#   that might  have  been  "Defined"  under  other circumstances  but
#   isn't.
def register (env):
    # Add the Subst builder
    a = env.Action (__build, "$$SUBSTR")
    b = env.Builder (action = a,
                     src_suffix = ".in",
                     emitter = __emit,
                     single_source = True)
    env.Append (BUILDERS = { "Subst": b })
    if env["verbose"]:
        env["SUBSTR"] = tools.makeVerboseCommandString ("Setting variables in ${SOURCE.srcpath} with:",
                                                        "Subst (${TARGET}, ${SOURCE})")
    else:
        env["SUBSTR"] = tools.makeStandardCommandString ("Setting variables in ${TARGET.file}...")

    # Add the Defined and Undefined methods
    env.AddMethod (__Defined, "Defined")
    env.AddMethod (__Undefined, "Undefined")
    env.AddMethod (CheckDefined, "CheckDefined")
