# Each target may define the following  environment variables that may
# be added to the corresponding construction variable:
#
# - CCFLAGS_ANS: Extra options to enable strict ANSI compliance;
# - LINKFLAGS_WIN:  Extra options  to have windows applications detach
#   themselves from the console.
import os
import re
import sconstools as tools
import sys

_pyFile = re.compile ("\\.py$", re.I)
_baseDir = os.path.dirname (__file__)

_configEnvVars = [ "CFLAGS", "CPPFLAGS", "CXXFLAGS", "LDFLAGS", "LIBS" ]

def __import(name):
    mod = __import__ (name)
    components = name.split ('.')
    for comp in components[1:]:
        mod = getattr (mod, comp)
    return mod

_targets = { "native": None }
_targetsPath = os.path.join (_baseDir, "targets")
if os.path.isdir (_targetsPath):
    for _target in os.listdir (_targetsPath):
        if _target[0] != "_" and re.search (_pyFile, _target):
            _name = os.path.splitext (_target)[0]
            _t = __import ("sconstools.targets.%s" % _name)
            _t.register (_targets)
else:
    print "WARNING: Targets path not found!"

def setNativeVariables (env, vars):
    p = sys.platform
    env["BUILD_TARGET"] = p
    if os.path.exists (os.path.join (_baseDir, "native", "%s.py" % p)):
        o = __import ("sconstools.native.%s" % p)
        o.setVariables (env, vars)
    else:
        # Ensure that all necessary variables are present
        env.Append (CCFLAGS = [],
                    CPPPATH = [],
                    CPPDEFINES = [],
                    LIBS = [],
                    LIBPATH = [],
                    LINKFLAGS = [])
        print "WARNING: Native variables for platform '%s' not found" % p

def addVariables (vars):
    targets = _targets.keys()
    targets.sort()
    from SCons.Variables import BoolVariable, EnumVariable
    vars.AddVariables (EnumVariable ("optim",
"""Select optimization settings. Available values are:
        - debug: Sets debug variables;
        - speed: Sets optimizations for speed;
        - size: Sets optimizations for size;
        - custom: Use variables from the environment:
        %s
""" % _configEnvVars,
                                     "custom",
                                     allowed_values = ("debug", "speed", "size", "custom")),
                     BoolVariable ("verbose",
                                   "If true, will print the complete command lines",
                                   False))
    if len (targets) > 1:
        vars.Add (EnumVariable ("target",
                                "Compilation target",
                                "native",
                                targets))

def setTarget (env, vars = None):
    if env.has_key ("target"):
        try:
            _t = _targets[env["target"]]
        except KeyError:
            print "WARNING: Unknown target, assuming native"
            _t = None
        if _t:
            env["BUILD_TARGET"] = env["target"]
            _t (env, vars)
        else:
            setNativeVariables (env, vars)
    else:
        setNativeVariables (env, vars)

    if env["optim"] == "custom":
        env.Replace (CCFLAGS = [],
                     CPPPATH = [],
                     CPPDEFINES = [],
                     LIBS = [],
                     LIBPATH = [],
                     LINKFLAGS = [])
        env.MergeFlags ([ os.environ.get (var, '')
                          for var in _configEnvVars ])

    commandStrings = [ "ARCOM", "ASCOM", "ASPPCOM", "CCCOM",
                       "CXXCOM", "LINKCOM", "RANLIBCOM", "RCCOM",
                       "SHCCCOM", "SHCXXCOM", "SHLINKCOM",
                       "SWIGCOM", "TARCOM", "ZIPCOM" ] 
    if not env["verbose"]:
        env["ARCOMSTR"]     = "Creating static library ${TARGET}..."
        env["ASCOMSTR"]     = "Assembling ${TARGET}..."
        env["ASPPCOMSTR"]   = "Preprocessing and assembling ${TARGET}..."
        env["CCCOMSTR"]     = "Compiling static ${TARGET} from C..."
        env["CXXCOMSTR"]    = "Compiling static ${TARGET} from C++..."
        env["LINKCOMSTR"]   = "Linking ${TARGET}..."
        env["RANLIBCOMSTR"] = "Indexing library ${TARGET}..."
        env["RCCOMSTR"]     = "Compiling resources ${TARGET}..."
        env["SHCCCOMSTR"]   = "Compiling shared ${TARGET} from C..."
        env["SHCXXCOMSTR"]  = "Compiling shared ${TARGET} from C++..."
        env["SHLINKCOMSTR"] = "Linking ${TARGET}..."
        env["SWIGCOMSTR"]   = "Generating interface file ${TARGET}..."
        env["TARCOMSTR"]    = "Archiving ${TARGET}..."
        env["ZIPCOMSTR"]    = "Zipping ${TARGET}..."
        for var in commandStrings:
            env["%sSTR" % var] = tools.makeStandardCommandString \
                                 (env["%sSTR" % var])
        env["INSTALLSTR"]   = tools.makeStandardCommandString \
                              ("Installing ${TARGET}...")
        # Workaround for a SCons bug: when using the internal zip
        # function, the zip command string is ignored
        import SCons.Tool.zip
        SCons.Tool.zip.zipAction.cmdstr = env["ZIPCOMSTR"]
    else:
        for var in commandStrings:
            env["%sSTR" % var] = tools.makeVerboseCommandString \
                                 ("Building ${TARGET} with:\n", "${%s}" % var)
        def _installStr (target, source, env):
            target = str (target)
            source = str (source)
            if os.path.isdir (source):
                type = 'directory'
            else:
                type = 'file'
            return 'Install %s: "%s" as "%s"' % (type, source, target)
        env["_installStr"]  = _installStr
        env["INSTALLSTR"]   = tools.makeStandardCommandString \
                              ("${_installStr (TARGET, SOURCE, __env__)}")
        # Workaround for a SCons bug: when using the internal zip
        # function, the zip command string is ignored
        import SCons.Tool.zip
        origStrFunction = SCons.Tool.zip.zipAction.strfunction
        def zipStrFunction (target, source, env):
            return tools.makeVerboseCommandString \
                   ("Building %s with:\n" % target[0],
                    origStrFunction (target, source, env))
        SCons.Tool.zip.zipAction.strfunction = zipStrFunction

    # Setup compiler colorizer
    if env["CC"]:
        colorCC = "color%s" % os.path.basename (env["CC"])
        if env.WhereIs (colorCC):
            env["CC"] = colorCC
    if env["CXX"]:
        colorCXX = "color%s" % os.path.basename (env["CXX"])
        if env.WhereIs (colorCXX):
            env["CXX"] = colorCXX

    # Setup Clang analyzer
    if env["ENV"].has_key ("CCC_ANALYZER_ANALYSIS"):
        env["ENV"]["CCC_CC"]  = env["CC"]
        env["ENV"]["CCC_CXX"] = env["CXX"]
        env["CC"]  = env["ENV"]["CC"]
        env["CXX"] = env["ENV"]["CXX"]
